package com.op.common.datascope.aspect;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import com.op.common.datascope.annotation.DataScope;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.op.common.core.constant.CacheConstants;
import com.op.common.core.context.SecurityContextHolder;
import com.op.common.core.text.Convert;
import com.op.common.core.utils.StringUtils;
import com.op.common.core.web.domain.BaseEntity;

import com.op.common.datascope.annotation.PoolDeptAuto;
import com.op.common.redis.service.RedisService;
import com.op.common.security.utils.SecurityUtils;
import com.op.system.api.domain.SysRole;
import com.op.system.api.domain.SysUser;
import com.op.system.api.model.LoginUser;

/**
 * 数据过滤处理
 *
 * @author sf
 */
@Aspect
@Component
public class DataScopeAspect {

	private static final Logger log = LoggerFactory.getLogger(DataScopeAspect.class);

	@Autowired
	private RedisService redisService;

	/**
	 * 全部数据权限
	 */
	public static final String DATA_SCOPE_ALL = "1";

	/**
	 * 自定数据权限
	 */
	public static final String DATA_SCOPE_CUSTOM = "2";

	/**
	 * 部门数据权限
	 */
	public static final String DATA_SCOPE_DEPT = "3";

	/**
	 * 部门及以下数据权限
	 */
	public static final String DATA_SCOPE_DEPT_AND_CHILD = "4";

	/**
	 * 仅本人数据权限
	 */
	public static final String DATA_SCOPE_SELF = "5";

	/**
	 * 数据权限过滤关键字
	 */
	public static final String DATA_SCOPE = "dataScope";

	@Before("@annotation(controllerDataScope)")
	public void doBefore(JoinPoint point, DataScope controllerDataScope) throws Throwable {
		clearDataScope(point);
		handleDataScope(point, controllerDataScope);
	}

	@Before("@annotation(controllerPoolDeptAuto)")
	public void doBefore(JoinPoint point, PoolDeptAuto controllerPoolDeptAuto)
			throws SecurityException, IllegalArgumentException, IllegalAccessException, NoSuchFieldException {

		Object obj = point.getArgs()[0];
		if (controllerPoolDeptAuto.pool()) {
			log.debug("已开启自动设置数据源标识");
			Field field = null;
			try {
				field = obj.getClass().getSuperclass().getDeclaredField("poolName");
			} catch (NoSuchFieldException e) {
				field = obj.getClass().getDeclaredField("poolName");
				log.error("If PoolDeptAuto annotation is used, the parameter must contain poolName!", e);
				throw e;
			}
			field.setAccessible(true);
			Object poolName = field.get(obj);
			if (poolName == null) {
				// 缓存用户工厂数据源
				poolName = redisService
						.getCacheObject(CacheConstants.USER_POOL_NAME_CURRENT + SecurityUtils.getUsername());
				if (StringUtils.isNotNull(poolName)) {
					field.set(obj, poolName);
				} else {
					log.error("Cannot be found the current poolName!");
				}
			}
		}
		if (controllerPoolDeptAuto.dept()) {
			Field field = null;
			try {
				field = obj.getClass().getSuperclass().getDeclaredField("deptId");
			} catch (NoSuchFieldException e) {
				field = obj.getClass().getDeclaredField("deptId");
				log.error("If PoolDeptAuto annotation is used, the parameter must contain deptId!", e);
				throw e;
			}
			field.setAccessible(true);
			Object deptId = field.get(obj);
			if (deptId == null) {
				// 获取当前的用户
				LoginUser loginUser = SecurityUtils.getLoginUser();
				if (StringUtils.isNotNull(loginUser)) {
					field.set(obj, loginUser.getSysUser().getDeptId());
				} else {
					log.error("Cannot be found the current login user deptId!");
				}
			}
		}
	}

	protected void handleDataScope(final JoinPoint joinPoint, DataScope controllerDataScope) {
		// 获取当前的用户
		LoginUser loginUser = SecurityUtils.getLoginUser();
		if (StringUtils.isNotNull(loginUser)) {
			SysUser currentUser = loginUser.getSysUser();
			// 如果是超级管理员，则不过滤数据
			if (StringUtils.isNotNull(currentUser) && !currentUser.isAdmin()) {
				String permission = StringUtils.defaultIfEmpty(controllerDataScope.permission(),
						SecurityContextHolder.getPermission());
				dataScopeFilter(joinPoint, currentUser, controllerDataScope.deptAlias(),
						controllerDataScope.userAlias(), permission);
			}
		}
	}

	/**
	 * 数据范围过滤
	 *
	 * @param joinPoint  切点
	 * @param user       用户
	 * @param deptAlias  部门别名
	 * @param userAlias  用户别名
	 * @param permission 权限字符
	 */
	public static void dataScopeFilter(JoinPoint joinPoint, SysUser user, String deptAlias, String userAlias,
									   String permission) {
		StringBuilder sqlString = new StringBuilder();
		List<String> conditions = new ArrayList<String>();

		for (SysRole role : user.getRoles()) {
			String dataScope = role.getDataScope();
			if (!DATA_SCOPE_CUSTOM.equals(dataScope) && conditions.contains(dataScope)) {
				continue;
			}
			if (StringUtils.isNotEmpty(permission) && StringUtils.isNotEmpty(role.getPermissions())
					&& !StringUtils.containsAny(role.getPermissions(), Convert.toStrArray(permission))) {
				continue;
			}
			if (DATA_SCOPE_ALL.equals(dataScope)) {
				sqlString = new StringBuilder();
				conditions.add(dataScope);
				break;
			} else if (DATA_SCOPE_CUSTOM.equals(dataScope)) {
				sqlString.append(StringUtils.format(
						" OR {}.dept_id IN ( SELECT dept_id FROM sys_role_dept WHERE role_id = {} ) ", deptAlias,
						role.getRoleId()));
			} else if (DATA_SCOPE_DEPT.equals(dataScope)) {
				sqlString.append(StringUtils.format(" OR {}.dept_id = {} ", deptAlias, user.getDeptId()));
			} else if (DATA_SCOPE_DEPT_AND_CHILD.equals(dataScope)) {
				sqlString.append(StringUtils.format(
						" OR {}.dept_id IN ( SELECT dept_id FROM sys_dept WHERE dept_id = {} or find_in_set( {} , ancestors ) )",
						deptAlias, user.getDeptId(), user.getDeptId()));
			} else if (DATA_SCOPE_SELF.equals(dataScope)) {
				if (StringUtils.isNotBlank(userAlias)) {
					sqlString.append(StringUtils.format(" OR {}.user_id = {} ", userAlias, user.getUserId()));
				} else {
					// 数据权限为仅本人且没有userAlias别名不查询任何数据
					sqlString.append(StringUtils.format(" OR {}.dept_id = 0 ", deptAlias));
				}
			}
			conditions.add(dataScope);
		}

		// 多角色情况下，所有角色都不包含传递过来的权限字符，这个时候sqlString也会为空，所以要限制一下,不查询任何数据
		if (StringUtils.isEmpty(conditions)) {
			sqlString.append(StringUtils.format(" OR {}.dept_id = 0 ", deptAlias));
		}

		if (StringUtils.isNotBlank(sqlString.toString())) {
			Object params = joinPoint.getArgs()[0];
			if (StringUtils.isNotNull(params) && params instanceof BaseEntity) {
				BaseEntity baseEntity = (BaseEntity) params;
				baseEntity.getParams().put(DATA_SCOPE, " AND (" + sqlString.substring(4) + ")");
			}
		}
	}

	/**
	 * 拼接权限sql前先清空params.dataScope参数防止注入
	 */
	private void clearDataScope(final JoinPoint joinPoint) {
		Object params = joinPoint.getArgs()[0];
		if (StringUtils.isNotNull(params) && params instanceof BaseEntity) {
			BaseEntity baseEntity = (BaseEntity) params;
			baseEntity.getParams().put(DATA_SCOPE, "");
		}
	}
}
